import java.io.*;
import java.sql.*;
import java.text.*;
// Add these imports for access to the required Oracle classes
import oracle.jdbc.driver.*;
import oracle.sql.BLOB;

public class TestBLOBGetBinaryOutputStream {
  Connection conn;
  
  public TestBLOBGetBinaryOutputStream() {
    try {
      DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
      conn = DriverManager.getConnection(
       "jdbc:oracle:thin:@dssw2k01:1521:orcl", "scott", "tiger");
    }
    catch (SQLException e) {
      System.err.println(e.getMessage());
      e.printStackTrace();
    }
  }
  
  public static void main(String[] args) 
   throws Exception, IOException {
    new TestBLOBGetBinaryOutputStream().process();
  }

  public void process() throws IOException, SQLException {
    int             rows      = 0;
    FileInputStream fin       = null;
    OutputStream    out       = null;
    ResultSet       rslt      = null;
    Statement       stmt      = null;
    BLOB            photo     = null;  // NOTE: oracle.sql.BLOB!!!
    long            person_id = 0;
    try {
      conn.setAutoCommit(false);

      // get Tim's person_id 
      stmt = conn.createStatement();
      rslt = stmt.executeQuery(
       "select person_id " + 
       "from   person " + 
       "where  last_name  = 'O''Reilly' " + 
       "and    first_name = 'Tim'");   
      while (rslt.next()) {
        rows++;
        person_id = rslt.getLong(1);
      }
      if (rows > 1) {
        System.err.println("Too many rows!");
        System.exit(1);
      }
      else if (rows == 0) {
        System.err.println("Not found!");
        System.exit(1);
      }
      rslt.close();
      rslt = null;

      // check to see if the row already exists
      rows = 0;
      rslt = stmt.executeQuery(
       "select photo " +
       "from   person_information " +
       "where  person_id = " + Long.toString( person_id ) + " " +
       "for update nowait");
      while (rslt.next()) {
        rows++;
        photo = (BLOB)rslt.getBlob(1);
      }
      rslt.close();
      rslt = null;
      
      // if it doesn't exist then insert
      // a row in the information table.
      // This creates the LOB locators
      if (rows == 0) {
        rows = stmt.executeUpdate(
         "insert into person_information " + 
         "( person_id, biography, photo ) " + 
         "values " +
         "( " + Long.toString( person_id ) + 
         ", empty_clob(), empty_blob() )");
        System.out.println(rows + " rows inserted");
          
        // Now retrieve the locator
        rows = 0;
        rslt = stmt.executeQuery(
         "select photo " +
         "from   person_information " +
         "where  person_id = " + Long.toString( person_id ) + " " +
         "for update nowait");
        rslt.next();
        photo = ((OracleResultSet)rslt).getBLOB(1);
        rslt.close();
        rslt = null;
      }
      stmt.close();
      stmt = null;

      // Now that we have the locator, 
      // lets store the photo
      File binaryFile = new File("tim.gif");
      fin = new FileInputStream(binaryFile);
      out = photo.getBinaryOutputStream();
      // Get the optimal buffer size from the BLOB
      byte[] buffer = new byte[photo.getBufferSize()];
      int length = 0;
      while ((length = fin.read(buffer)) != -1) {
        out.write(buffer, 0, length);
      }
      // You've got to close the output stream before
      // you commit, or the changes are lost! 
      out.close();  
      out = null;
      fin.close();
      fin = null;
      conn.commit();
    }
    catch (SQLException e) {
      System.err.println("SQL Error: " + e.getMessage());
    }
    catch (IOException e) {
      System.err.println("IO Error: " + e.getMessage());
    }
    finally {
      if (rslt != null) 
        try { rslt.close(); } catch (SQLException ignore) { }
      if (stmt != null) 
        try { stmt.close(); } catch (SQLException ignore) { }
      if (out != null) 
        try { out.close(); }  catch (IOException ignore) { }
      if (fin != null) 
        try { fin.close(); }  catch (IOException ignore) { }
    }
  }

  protected void finalize() 
   throws Throwable {
    if (conn != null) 
      try { conn.close(); } catch (SQLException ignore) { }
    super.finalize(); 
  }
}
